import requests

from typing import List
from typing import Optional
from typing import Union

from ratelimit import limits
from .utils import sleep_and_retry

from .objects import NuclinoObject
from .objects import get_loader


BASE_URL = 'https://api.nuclino.com/v0'


class NuclinoError(Exception):
    pass


def join_url(base_url, path):
    return '/'.join(part.strip('/') for part in [base_url, path])


class Client:
    '''
    Base class for Nuclino API client. May be used as a context processor.
    '''

    def __init__(
        self,
        api_key: str,
        base_url: Optional[str] = BASE_URL,
        requests_per_minute: int = 140
    ):
        '''
        :param api_key:       your Nuclino API key.
        :param base_url:      base url to send API requests.
        :requests_per_minute: max requests per minute. If limit exceeded, client will wait
                              for some time before processing the next request.
        '''
        self.check_limit = sleep_and_retry()(
            limits(requests_per_minute, period=60)(lambda: None)
        )
        self.session = requests.Session()
        self.session.headers['Authorization'] = api_key
        self.timer = None
        self.base_url = base_url

    def __enter__(self):
        return self

    def __exit__(self, *_):
        self.close()

    def close(self):
        self.session.close()

    def _process_response(
        self,
        response: requests.models.Response
    ) -> Union[List, NuclinoObject, dict]:
        '''
        General method that processes API responses. Raises error on HTTP
        errors, sends results to parser on 200 ok.

        :param response: response object, received after calling API.
        '''
        content = response.json()
        if response.status_code != 200:
            message = content.get('message', '')
            raise NuclinoError(f'{response.status_code}: {message}')
        else:
            data = content['data']
            return self.parse(data)

    def parse(self, source: dict) -> Union[List, NuclinoObject, dict]:
        '''
        Parse successful response dictionary. This method will determine the
        type of object, that was returned, and construct corresponding
        NuclinoObject as the return result.

        :param source: the "data" dictionary from Nuclino API response.
        :returns:      corresponsing NuclinoObject constructed from `source`.
        '''

        if 'object' not in source:
            return source
        func = get_loader(source['object'])
        result = func(source, self)
        if isinstance(result, NuclinoObject):
            return result
        elif isinstance(result, list):
            return [self.parse(li) for li in result]
        else:
            return source

    def get(self, path: str, params: dict = {}) -> Union[List, NuclinoObject, dict]:
        self.check_limit()
        response = self.session.get(join_url(self.base_url, path), params=params)
        return self._process_response(response)

    def delete(self, path: str) -> Union[List, NuclinoObject, dict]:
        self.check_limit()
        response = self.session.delete(join_url(self.base_url, path))
        return self._process_response(response)

    def post(self, path: str, data: dict) -> Union[List, NuclinoObject, dict]:
        headers = {'Content-Type': 'application/json'}
        self.check_limit()
        response = self.session.post(
            join_url(self.base_url, path),
            json=data,
            headers=headers
        )
        return self._process_response(response)

    def put(self, path: str, data: dict) -> Union[List, NuclinoObject, dict]:
        headers = {'Content-Type': 'application/json'}
        self.check_limit()
        response = self.session.put(
            join_url(self.base_url, path),
            json=data,
            headers=headers
        )
        return self._process_response(response)


class Nuclino(Client):

    def get_user(self, user_id: str) -> Union[List, NuclinoObject, dict]:
        '''
        Get information associated with a specific user by ID.

        :param user_id: ID of the user to get.

        :returns: a User object
        '''

        path = f'/users/{user_id}'
        return self.get(path)

    def get_teams(
        self,
        limit: Optional[int] = None,
        after: Optional[str] = None
    ) -> Union[List, NuclinoObject, dict]:
        '''
        Get list of teams available for user.

        :param limit: number between 1 and 100 to limit the results.
        :param after: only return teams that come after the given team ID.

        :returns: list of Team objects.
        '''

        path = '/teams'
        params = {}
        if limit is not None:
            params['limit'] = str(limit)
        if after is not None:
            params['after'] = after
        return self.get(path, params)

    def get_team(self, team_id: str):
        '''
        Get specific team by ID.

        :param team_id: ID of the team to get.

        :returns: Team object.
        '''

        path = f'/teams/{team_id}'
        return self.get(path)

    def get_workspaces(
        self,
        team_id: Optional[str] = None,
        limit: Optional[int] = None,
        after: Optional[str] = None
    ) -> Union[List, NuclinoObject, dict]:
        '''
        Get list of workspaces available for user.

        :param team_id: ID of the team the returned workspaces should belong to.
        :param limit: number between 1 and 100 to limit the results.
        :param after: only return workspaces that come after the given workspace
                      ID.

        :returns: list of Workspace objects.
        '''

        path = '/workspaces'
        params = {}
        if team_id is not None:
            params['teamId'] = team_id
        if limit is not None:
            params['limit'] = str(limit)
        if after is not None:
            params['after'] = after
        return self.get(path, params)

    def get_workspace(self, workspace_id: str) -> Union[List, NuclinoObject, dict]:
        '''
        Get specific workspace by ID.

        :param workspace_id: ID of the workspace to get.

        :returns: Workspace object.
        '''

        path = f'/workspaces/{workspace_id}'
        return self.get(path)

    def get_items(
        self,
        team_id: Optional[str] = None,
        workspace_id: Optional[str] = None,
        limit: Optional[int] = None,
        after: Optional[str] = None,
        search: Optional[str] = None
    ) -> Union[List, NuclinoObject, dict]:
        '''
        Get list of items and collection from the team or the workspace. Either
        `team_id` or `workspace_id` parameter is required. This method is also
        used for item search, use `search` parameter.

        :param team_id: ID of the team the returned items should belong to.
        :param team_id: ID of the workspace the returned items should belong to.
        :param limit:   number between 1 and 100 to limit the results.
        :param after:   only return workspaces that come after the given workspace
                        ID.
        :param search:  search query.

        :returns: list of Item and Collection objects.
        '''

        path = '/items'
        params = {}
        if team_id is not None:
            params['teamId'] = team_id
        if workspace_id is not None:
            params['workspaceId'] = workspace_id
        if limit is not None:
            params['limit'] = str(limit)
        if after is not None:
            params['after'] = after
        if search is not None:
            params['search'] = search
        return self.get(path, params)

    def get_item(self, item_id: str) -> Union[List, NuclinoObject, dict]:
        '''
        Get specific item or collection by ID.

        :param item_id: ID of the item to get.

        :returns: Item or Collection object.
        '''

        path = f'/items/{item_id}'
        return self.get(path)

    def get_collection(self, collection_id: str) -> Union[List, NuclinoObject, dict]:
        '''
        Alias for get_item. Get specific item or collection by ID.

        :param item_id: ID of the item to get.

        :returns: Item or Collection object.
        '''

        return self.get_item(collection_id)

    def create_item(
        self,
        workspace_id: Optional[str] = None,
        parent_id: Optional[str] = None,
        object: Optional[str] = 'item',
        title: Optional[str] = None,
        content: Optional[str] = None,
        index: Optional[int] = None
    ) -> Union[List, NuclinoObject, dict]:
        '''
        Create a new item or collection in the workspace or as a child of a
        collection. Either `workspace_id` or `parent_id` parameter is required.

        :param workspace_id: ID of the workspace the item should be put in (will
                             be placed at the root of the workspace).
        :param parent_id:    ID of the collection the item should be put in.
        :param object:       'item' or 'collection'.
        :param title:        item or collection title.
        :param content:      item content (only for items).
        :param index:        where to put this item in the tree. If not
                             specified — will be put at the end.

        :returns: the created Item or Collection object.
        '''

        path = '/items'
        data = {'object': object}
        if workspace_id is not None:
            data['workspaceId'] = workspace_id
        if parent_id is not None:
            data['parentId'] = parent_id
        if title is not None:
            data['title'] = title
        if content is not None:
            data['content'] = content
        if index is not None:
            data['index'] = str(index)
        return self.post(path, data)

    def create_collection(
        self,
        workspace_id: Optional[str] = None,
        parent_id: Optional[str] = None,
        title: Optional[str] = None,
        index: Optional[int] = None
    ) -> Union[List, NuclinoObject, dict]:
        '''
        Create a collection in the workspace or as a child of another collection.
        Either `workspace_id` or `parent_id` parameter is required.

        :param workspace_id: ID of the workspace the collection should be put in
                             (will be placed at the root of the workspace).
        :param parent_id:    ID of the collection this collection should be put in.
        :param title:        collection title.
        :param index:        where to put this collection in the tree. If not
                             specified — will be put at the end.

        :returns: the created Collection object.
        '''

        return self.create_item(
            workspace_id=workspace_id,
            parent_id=parent_id,
            object='collection',
            title=title,
            content=None,
            index=index
        )

    def update_item(
        self,
        item_id: str,
        title: Optional[str] = None,
        content: Optional[str] = None
    ) -> Union[List, NuclinoObject, dict]:
        '''
        Update item or collection.

        :param item_id: ID of the item to update.
        :param title:   new item title. If not specified — won't be changed.
        :param content: new item content (only for items). If not specified —
                        won't be changed.

        :returns: updated Item or Collection object.
        '''

        path = f'/items/{item_id}'
        data = {}
        if title is not None:
            data['title'] = title
        if content is not None:
            data['content'] = content
        return self.put(path, data=data)

    def update_collection(
        self,
        collection_id: str,
        title: Optional[str] = None
    ) -> Union[List, NuclinoObject, dict]:
        '''
        Update collection title.

        :param collection_id: ID of the collection to update.
        :param title:      new collection title. If not specified — won't be
                           changed.

        :returns: updated Collection object.
        '''

        return self.update_item(collection_id, title=title, content=None)

    def delete_item(self, item_id: str) -> Union[List, NuclinoObject, dict]:
        '''
        Move item or collection to trash.

        :param item_id: ID of the item to delete.

        :returns: a dictionary with ID of deleted item.
        '''
        path = f'/items/{item_id}'
        return self.delete(path)

    def delete_collection(self, collection_id: str) -> Union[List, NuclinoObject, dict]:
        '''
        Alias for delete_item. Move item or collection to trash.

        :param item_id: ID of the item to delete.

        :returns: a dictionary with ID of deleted item.
        '''

        return self.delete_item(collection_id)

    def get_file(self, file_id: str) -> Union[List, NuclinoObject, dict]:
        '''
        Get a file object by ID.

        :param item_id: ID of the file to get.

        :returns: a File object.
        '''

        path = f'/files/{file_id}'
        return self.get(path)



from __future__ import annotations
from typing import List, Union, Optional, Callable


class NuclinoObject:
    _object = ''

    @classmethod
    def load(cls, props: dict, nuclino):
        return cls(props, nuclino)

    def __init__(
        self,
        props: dict,
        nuclino
    ):
        self.data = dict(props)
        self.nuclino = nuclino

class User(NuclinoObject):
    _object = "user"

    def __init__(self, props: dict, nuclino):
        super().__init__(props, nuclino)

        self.id = props['id']
        self.first_name = props['firstName']
        self.last_name = props['lastName']
        self.email = props['email']
        self.avatar_url = props.get('avatarUrl')

    def __repr__(self) -> str:
        return f'<User "{self.first_name} {self.last_name}">'


class Team(NuclinoObject):
    _object = "team"

    def __init__(
        self,
        props: dict,
        nuclino
    ):
        super().__init__(props, nuclino)

        self.id = props['id']
        self.url = props['url']
        self.name = props['name']
        self.created_at = props['createdAt']
        self.created_user_id = props['createdUserId']

    def get_workspaces(self) -> List[Workspace]:
        '''
        Make an API call to get all workspaces that belong to this team.

        :returns: list of Workspace objects.
        '''

        return self.nuclino.get_workspaces(team_id=self.id)

    def __repr__(self) -> str:
        return f'<Team "{self.name}">'


class Workspace(NuclinoObject):
    _object = "workspace"

    def __init__(
        self,
        props: dict,
        nuclino
    ):
        super().__init__(props, nuclino)

        self.id = props['id']
        self.team_id = props['teamId']
        self.name = props['name']
        self.created_at = props['createdAt']
        self.created_user_id = props['createdUserId']
        self.child_ids = props['childIds']

    def get_team(self) -> Team:
        '''
        Make an API call to get the team this workspace belongs to.

        :returns: Team object.
        '''

        return self.nuclino.get_team(self.team_id)

    def get_children(self) -> List[Union[Item, Collection]]:
        return [self.nuclino.get_item(id_) for id_ in self.child_ids]

    def create_item(
        self,
        object: Optional[str] = 'item',
        title: Optional[str] = None,
        content: Optional[str] = None,
        index: Optional[int] = None
    ) -> Union[Item, Collection]:
        '''
        Create a new item or collection under this workspace.

        :param object:  'item' or 'collection'.
        :param title':  item title.
        :param content: item content (only for items).
        :param index:   where to put this item in the tree. If not
                        specified — will be put at the end.

        :returns: Item or Collection object.
        '''

        return self.nuclino.create_item(
            workspace_id=self.id,
            object=object,
            title=title,
            content=content,
            index=index
        )

    def create_collection(
        self,
        title: Optional[str] = None,
        index: Optional[int] = None
    ) -> Union[Item, Collection]:
        '''
        Create a new collection under this workspace.

        :param title':  collection title.
        :param index:   where to put this collection in the tree. If not
                        specified — will be put at the end.

        :returns: Collection object.
        '''

        return self.nuclino.create_item(
            workspace_id=self.id,
            object="collection",
            title=title,
            content=None,
            index=index
        )

    def __repr__(self) -> str:
        return f'<Workspace "{self.name}">'


class Collection(NuclinoObject):
    _object = "collection"

    def __init__(
        self,
        props: dict,
        nuclino
    ):
        super().__init__(props, nuclino)

        self.id = props['id']
        self.child_ids = props['childIds']
        self.created_at = props['createdAt']
        self.created_user_id = props['createdUserId']
        self.last_updated_at = props['lastUpdatedAt']
        self.last_updated_user_id = props['lastUpdatedUserId']
        self.title = props['title']
        self.url = props['url']
        self.workspace_id = props['workspaceId']

    def get_children(self) -> List[Union[Item, Collection]]:
        '''
        Make an API call to get the list of direct children of this collection.

        :returns: list of Item and Collection objects.
        '''

        return [self.nuclino.get_item(id_) for id_ in self.child_ids]

    def get_workspace(self) -> Workspace:
        '''
        Make an API call to get the workspace this collection belongs to.

        :returns: Workspace object.
        '''

        return self.nuclino.get_workspace(self.workspace_id)

    def create_item(
        self,
        object: Optional[str] = 'item',
        title: Optional[str] = None,
        content: Optional[str] = None,
        index: Optional[int] = None
    ) -> Union[Item, Collection]:
        '''
        Create an item or a collection under this collection.

        :param object:  'item' or 'collection'.
        :param title:   item title.
        :param content: item content.
        :param index:   where to put this item in the tree. If not
                        specified — will be put at the end.

        :returns: created Item or Collection object.
        '''

        return self.nuclino.create_item(
            parent_id=self.id,
            object=object,
            title=title,
            content=content,
            index=index
        )

    def create_collection(
        self,
        title: Optional[str] = None,
        content: Optional[str] = None,
        index: Optional[int] = None
    ) -> Union[Item, Collection]:
        '''
        Create another collection under this collection.

        :param title:   collection title.
        :param index:   where to put this collection in the tree. If not
                        specified — will be put at the end.

        :returns: created Collection object.
        '''

        return self.nuclino.create_item(
            parent_id=self.id,
            object="collection",
            title=title,
            index=index
        )

    def delete(self) -> dict:
        '''
        Move this collection to trash.

        :returns: dictionary with this collection ID.
        '''

        return self.nuclino.delete_collection(self.id)

    def update(
        self,
        title: Optional[str] = None
    ):
        '''
        Change this collection title.

        :param title: new title value.

        :returns: updated Collection object.
        '''

        return self.nuclino.update_collection(self.id, title)

    def __repr__(self) -> str:
        return f'<Collection "{self.title}">'


class Item(NuclinoObject):
    _object = "item"

    def __init__(
        self,
        props: dict,
        nuclino
    ):
        super().__init__(props, nuclino)

        self.id = props['id']
        self.content_meta = props['contentMeta']
        self.created_at = props['createdAt']
        self.created_user_id = props['createdUserId']
        self.last_updated_at = props['lastUpdatedAt']
        self.last_updated_user_id = props['lastUpdatedUserId']
        self.title = props['title']
        self.url = props['url']
        self.workspace_id = props['workspaceId']
        self.content = props.get('content')

    def get_workspace(self) -> Workspace:
        '''
        Make an API call to get the workspace this item belongs to.

        :returns: Workspace object.
        '''

        return self.nuclino.get_workspace(self.workspace_id)

    def get_items(self) -> List[Union[Item, Collection]]:
        '''
        Make API calls to get list of items or collections that are referenced in
        this item.

        :returns: list of Item or Collection objects.
        '''

        return [self.nuclino.get_item(id_) for id_ in self.content_meta['itemIds']]

    def get_files(self) -> List[File]:
        '''
        Make API calls to get the list of files attached to this file.

        :returns: list of File objects.
        '''

        return [self.nuclino.get_file(id_) for id_ in self.content_meta['fileIds']]

    def delete(self) -> dict:
        '''
        Move this item to trash.

        :returns: dictionary with this item id.
        '''

        return self.nuclino.delete_item(self.id)

    def update(
        self,
        title: Optional[str] = None,
        content: Optional[str] = None
    ):
        '''
        Update this item.

        :param title:   new item title.
        :param content: new item content.

        returns: updated Item object.
        '''

        return self.nuclino.update_item(self.id, title, content)

    def __repr__(self):
        return f'<Item "{self.title}">'


class File(NuclinoObject):
    _object = "file"

    def __init__(
        self,
        props: dict,
        nuclino
    ):
        super().__init__(props, nuclino)

        self.id = props['id']
        self.item_id = props['itemId']
        self.file_name = props['fileName']
        self.created_at = props['createdAt']
        self.created_user_id = props['createdUserId']
        self.download = props['download']

    def get_item(self) -> Item:
        '''
        Make an API call to get the item this file is attached to.

        :returns: Item object.
        '''

        return self.nuclino.get_item(self.item_id)

    def __repr__(self):
        return f'<file "{self.file_name}">'


def load_list(props: dict, nuclino):
    if props['object'] != 'list':
        raise RuntimeError("Wrong object")
    else:
        return props['results']


def get_loader(name: str) -> Callable:
    classes = {
        'list': load_list,
    }
    classes.update(
        {no._object: no.load for no in NuclinoObject.__subclasses__()}
    )
    return classes[name]



import json
import os

from nuclino.api import Client
from nuclino.api import NuclinoError
from nuclino import objects
from pathlib import Path
from unittest import TestCase
from unittest import mock


def open_data_file(name):
    data_root = Path(os.path.abspath(__file__)).parent / 'test_data'
    with open(data_root / name) as f:
        return json.load(f)


class TestProcessResponse(TestCase):
    def setUp(self):
        self.client = Client('api_key')

    def test_error(self):
        response = mock.Mock()
        response.status_code = 400
        data = {'message': 'Error message'}
        response.json.return_value = data

        with self.assertRaises(NuclinoError) as exc:
            self.client._process_response(response)
            self.assertIn(str(response.status_code), str(exc))
            self.assertIn(data['message'], str(exc))

    @mock.patch('nuclino.api.Client.parse')
    def test_normal(self, parse):
        response = mock.Mock()
        response.status_code = 200
        data = {
            "status": "success",
            "data": {
                "id": "1034eebc-b96a-42b7-92ea-6795d6440fda"
            }
        }
        response.json.return_value = data

        self.client._process_response(response)
        parse.assert_called_with(data['data'])


class TestParse(TestCase):
    def setUp(self):
        self.client = Client('api_key')

    def test_delete_response(self):
        data = {"id": "1034eebc-b96a-42b7-92ea-6795d6440fda"}
        result = self.client.parse(data)
        self.assertEqual(result, data)
    
    def test_user(self):
        data = open_data_file('get_user.json')
        result = self.client.parse(data['data'])
        self.assertIsInstance(result, objects.User)

    def test_team(self):
        data = open_data_file('get_team.json')
        result = self.client.parse(data['data'])
        self.assertIsInstance(result, objects.Team)

    def test_teams(self):
        data = open_data_file('get_teams.json')
        result = self.client.parse(data['data'])
        self.assertIsInstance(result, list)
        for t in result:
            self.assertIsInstance(t, objects.Team)

    def test_workspace(self):
        data = open_data_file('get_workspace.json')
        result = self.client.parse(data['data'])
        self.assertIsInstance(result, objects.Workspace)

    def test_workspaces(self):
        data = open_data_file('get_workspaces.json')
        result = self.client.parse(data['data'])
        self.assertIsInstance(result, list)
        for t in result:
            self.assertIsInstance(t, objects.Workspace)

    def test_item(self):
        data = open_data_file('get_item.json')
        result = self.client.parse(data['data'])
        self.assertIsInstance(result, objects.Item)

    def test_cluster(self):
        data = open_data_file('get_collection.json')
        result = self.client.parse(data['data'])
        self.assertIsInstance(result, objects.Collection)

    def test_items(self):
        data = open_data_file('get_items.json')
        result = self.client.parse(data['data'])
        self.assertIsInstance(result, list)
        for t in result:
            self.assertIsInstance(t, (objects.Item, objects.Collection))

    def test_file(self):
        data = open_data_file('get_file.json')
        result = self.client.parse(data['data'])
        self.assertIsInstance(result, objects.File)



import time
from functools import wraps
from ratelimit.exception import RateLimitException


def sleep_and_retry(quiet: bool = False):
    def decorator(func):
        '''
        This is sleep_and_retry decorator from ratelimit library but with added
        printed message so that user is aware of the delay.
        '''

        @wraps(func)
        def wrapper(*args, **kargs):
            while True:
                try:
                    return func(*args, **kargs)
                except RateLimitException as exception:
                    if not quiet:
                        print(f'Rate limited. Sleeping for {exception.period_remaining:.0f} seconds')
                    time.sleep(exception.period_remaining)
        return wrapper
    return decorator



'''
These tests call live API and require API_KEY env var to be set.
Beware of rate limiting while running these tests.
At the time of writing there's a limit of 150 requests per minute.
This test suite runs 36 requests.

WARNING: These tests create test entities (and then delete them) in
one of your workspaces.
'''

import os

from nuclino import objects
from nuclino.api import Nuclino
from unittest import TestCase


API_KEY = os.environ['API_KEY']


class TestCaseWithKey(TestCase):
    def setUp(self):
        self.client = Nuclino(API_KEY)

    def tearDown(self):
        self.client.close()


class TestTeams(TestCaseWithKey):
    def test_normal(self):
        result = self.client.get_teams()  # 1req
        for t in result:
            self.assertIsInstance(t, objects.Team)

    def test_limit(self):
        result = self.client.get_teams(limit=1)  # 1req
        self.assertEqual(len(result), 1)

    def test_after(self):
        teams = self.client.get_teams()  # 1req
        team_id = teams[0].id
        result = self.client.get_teams(after=team_id)  # 1req
        if result:
            self.assertFalse(result[0].id == team_id)

    def test_team(self):
        teams = self.client.get_teams()  # 1req
        team_id = teams[0].id
        result = self.client.get_team(team_id)
        self.assertIsInstance(result, objects.Team)


class TestUser(TestCaseWithKey):
    def test_normal(self):
        teams = self.client.get_teams() #1req
        user_id = teams[0].created_user_id
        result = self.client.get_user(user_id) #1req
        self.assertIsInstance(result, objects.User)


class TestWorkspaces(TestCaseWithKey):
    def test_normal(self):
        result = self.client.get_workspaces()  # 1req
        for t in result:
            self.assertIsInstance(t, objects.Workspace)

    def test_limit(self):
        result = self.client.get_workspaces(limit=1)  # 1req
        self.assertEqual(len(result), 1)

    def test_after(self):
        workspaces = self.client.get_workspaces()  # 1req
        workspace_id = workspaces[0].id
        result = self.client.get_workspaces(after=workspace_id)  # 1req
        if result:
            self.assertFalse(result[0].id == workspace_id)

    def test_workspace(self):
        workspaces = self.client.get_workspaces()  # 1req
        workspace_id = workspaces[0].id
        result = self.client.get_workspace(workspace_id)
        self.assertIsInstance(result, objects.Workspace)

    def test_team_id(self):
        teams = self.client.get_teams()  # 1req
        team_id = teams[0].id
        result = self.client.get_workspaces(team_id=team_id)  # 1req
        self.assertEqual(result[0].team_id, team_id)


class TestItems(TestCaseWithKey):
    def test_by_workspace(self):
        workspaces = self.client.get_workspaces()
        workspace_id = workspaces[0].id
        result = self.client.get_items(workspace_id=workspace_id)  # 1req
        for i in result:
            self.assertIsInstance(i, (objects.Item, objects.Collection))

    def test_by_team(self):
        teams = self.client.get_teams()
        team_id = teams[0].id
        result = self.client.get_items(team_id=team_id)  # 1req
        for i in result:
            self.assertIsInstance(i, (objects.Item, objects.Collection))

    def test_limit(self):
        workspaces = self.client.get_workspaces()
        workspace_id = workspaces[0].id
        result = self.client.get_items(workspace_id=workspace_id, limit=1)  # 1req
        self.assertEqual(len(result), 1)

    def test_after(self):
        workspaces = self.client.get_workspaces()
        workspace_id = workspaces[0].id

        items = self.client.get_items(workspace_id=workspace_id)  # 1req
        item_id = items[0].id
        result = self.client.get_items(workspace_id=workspace_id,
                                       after=item_id)  # 1req
        if result:
            self.assertFalse(result[0].id == item_id)

    def test_search(self):
        teams = self.client.get_teams()
        team_id = teams[0].id
        items = self.client.get_items(team_id=team_id)  # 1req
        filtered = self.client.get_items(team_id=team_id,
                                         search='welcome')  # 1req
        self.assertNotEqual(len(items), len(filtered))

    def test_item(self):
        teams = self.client.get_teams()
        team_id = teams[0].id
        items = self.client.get_items(team_id=team_id)  # 1req
        result = self.client.get_item(items[0].id)
        self.assertIsInstance(result, (objects.Item, objects.Collection))


class TestModifyItems(TestCaseWithKey):
    def test_create_item(self):
        workspaces = self.client.get_workspaces()  # 1req
        workspace_id = workspaces[0].id
        item = self.client.create_item(
            workspace_id=workspace_id,
            object='item',
            title='Test Title',
            content='Test content\n'
        )  # 1req
        self.assertIsInstance(item, objects.Item)
        item.delete()  # 1req

    def test_create_child(self):
        workspaces = self.client.get_workspaces()  # 1req
        workspace_id = workspaces[0].id
        collection = self.client.create_item(
            workspace_id=workspace_id,
            object='collection',
            title='Test Collection',
        )  # 1req
        self.assertIsInstance(collection, objects.Collection)

        item = self.client.create_item(
            parent_id=collection.id,
            object='item',
            title='Test Title',
            content='Test content\n'
        )  # 1req
        self.assertIsInstance(item, objects.Item)
        item.delete()  # 1req
        collection.delete()  # 1req

    def test_update_item(self):
        workspaces = self.client.get_workspaces()  # 1req
        workspace_id = workspaces[0].id
        item = self.client.create_item(
            workspace_id=workspace_id,
            object='item',
            title='Test Title',
            content='Test content\n'
        )  # 1req
        new_title = 'New Title'
        new_content = 'New Content\n'
        updated_item = self.client.update_item(item.id,
                                               title=new_title,
                                               content=new_content)  # 1req
        self.assertIsInstance(updated_item, objects.Item)
        self.assertEqual(updated_item.title, new_title)
        self.assertEqual(updated_item.content, new_content)
        item.delete()  # 1req

    def test_delete_item(self):
        workspaces = self.client.get_workspaces()  # 1req
        workspace_id = workspaces[0].id
        item = self.client.create_item(
            workspace_id=workspace_id,
            object='item',
            title='Test Title',
            content='Test content\n'
        )  # 1req
        item_id = item.id
        result = self.client.delete_item(item_id)
        self.assertEqual(result['id'], item_id)



from setuptools import setup


SHORT_DESCRIPTION = 'Nuclino API wrapper'

try:
    with open('README.md', encoding='utf8') as readme:
        LONG_DESCRIPTION = readme.read()

except FileNotFoundError:
    LONG_DESCRIPTION = SHORT_DESCRIPTION


setup(
    name='PyNuclino',
    python_requires='>=3.7',
    description=SHORT_DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type='text/markdown',
    version='0.2.1',
    author='Daniel Minukhin',
    author_email='ddddsa@gmail.com',
    url='https://github.com/Vanderhoof/PyNuclino',
    packages=['nuclino'],
    license='MIT',
    platforms='any',
    install_requires=[
        'ratelimit',
        'requests',
    ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Documentation",
        "Topic :: Text Processing :: Markup",
        "Topic :: Utilities",
    ]
)
